fl                                                               fl

{
MCP23017(16bits I/O Expander)
      
PropForth 5.5
2014/04/27 15:31:22

}

\ Register string-data in dictionary
: s, parsenw dup C@ 1+ bounds dup rot2 do C@++ c, loop drop ;

\ =========================================================================== 
\ Constants 
\ =========================================================================== 
\ Slave addres h20 for MCP23017
0 wconstant addr_input                  \ A2=A1=A0=0
h40 addr_input 1 lshift or wconstant MCP23017

\ reset pin
0 wconstant RST
hFFFFFFFE constant RST_mask

\ register
0 wconstant IODIR
2 wconstant IPOL
4 wconstant GPINTEN
6 wconstant DEFVAL
8 wconstant INTCON
hA wconstant IOCON
hC wconstant GPPU
hE wconstant INTF
h10 wconstant INTCAP
h12 wconstant GPIO
h14 wconstant OLAT

\ Define string data
wvariable reg_name -2 allot 
s, IODIRA s, IODIRB s, IPOLA s, IPOLB s, GPINTENA s, GPINTENB s, DEFVALA s, DEFVALB s, INTCONA 
s, INTCONB s, IOCON s, IOCON s, GPPUA s, GPPUB s, INTFA s, INTFB s, INTCAPA s, INTCAPB s, GPIOA 
s, GPIOB s, OLATA s, OLATB

\ =========================================================================== 
\ Main 
\ =========================================================================== 
\ Print out string inside string table
\ ( n1 n2 -- ) n1:index(0,1,2,..,n) n2:stringarray's address
: dispStr
swap dup 0 <>
if
     0 do dup C@ + 1+ loop
else
     drop
then
\ Print string
.cstr
;

\ Print tab
\ ( -- )
: tab 9 emit ;

\ Separate 16bits to 2bytes
\ ( n1 -- n2 n3 ) n1:16bit[hex]  n2:upper8bits n3:lower8bits
: separate_byte dup hFF00 and 8 rshift swap hFF and ;

\ Write on i2c and get t/f
\ ( n1 n2 -- n3 )  n1:data n2:t/f  n3:t/f
: swap_wr swap _eewrite or ;
                
\ Reset MCP23017
\ ( -- ) 
: reset 
RST pinout 
outa COG@ dup RST_mask and outa COG!    \ Set RST to low     
RST_mask invert or outa COG!            \ Set RST to hi
;

\ Display all registers(IOCON.BANK=0)
\ ( -- )   
: disp_reg
hex
." Register name" tab ." data[hex]" cr
IODIR MCP23017               \ ( n1 n2 )

\ Start I2C 
_eestart
\ Write slave address[wr], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)   
tuck _eewrite                 \ ( n2 n1 t/f )
\ Write register
swap_wr                       \ ( n2 t/f )
swap                          \ ( t/f n2 )
\ Start read_process
Sr
\ Write slave address[rd], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)
1 or _eewrite or              \ ( t/f )

d21 0 do
     \ Read 1byte ,then set sda to Hi(NACK:master->slave)
     0 _eeread
     i reg_name dispStr tab 
     i 4 5 between 0= if tab then 
     . cr            
loop
\ Read last byte
-1 _eeread
." OLATB" tab tab . cr
\ Stop I2C
_eestop
err?
cr
decimal                                         
;

\ Write sequence
\ ( n1 n2 n3 n4 -- t/f ) n1:upper8bits n2:lower8bits n3:register n4:slave address  
\ t/f:true if there is i2c-error
: wr_seq
\ Start I2C 
_eestart
\ Write slave address[wr], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)   
_eewrite                                \ ( n1 n2 n3 t/f )
\ Write register
swap_wr                                 \ ( t/f )
\ Write 2bytes
swap_wr                                 \ ( n1 t/f )
swap_wr                                 \ ( t/f )
\ Stop I2C
_eestop
;

\ Write data[16bits] to each registers[****A/****B]
\ ( n1 n2 -- )  n1:register n2:16bits[hex]
\ register
\ IODIRA       IODIRA(lower8bits)/IODIRB(upper8bits)  1=input 0=output
\ IPOLA        IPOLA(lower8bits)/IPOLB(upper8bits)  1=oppsitelogic state 0=same logic state
\ GPINTENA     GPINTENA(lower8bits)/GPINTENB(upper8bits)  1=enable 0=disable
\ DEFVALA      DEFVALA(lower8bits)/DEFVALB(upper8bits)  
\ INTCONA      INTCONA(lower8bits)/INTCONB(upper8bits)    
\              1=compared for interupt-on-change 0=compared against previous value
\ IOCON        IOCONA(lower8bits)/IOCONB(upper8bits)
\ GPPUA        GPPUA(lower8bits)/GPPUB(upper8bits)  1=Pull-up enabled  0= Pull-up disabled
\ GPIOA        GPIOA(lower8bits)/GPIOB(upper8bits)      1=logic-hi  0=logic-lo
\ OLATA        OLATA(lower8bits)/OLATB(upper8bits)      1=logic-hi  0=logic-lo
\ ( n1 n2 -- )  n1:register n2:16bits[hex]
: wr_MCP23017
separate_byte                           
rot MCP23017        \ ( upper8bits lower8bits n1 slave-address) 
wr_seq
err?
;
  


\ read sequence
\ ( n1 n2 -- n3 n4 t/f ) n1:register n2:slave address   n3:lower8bits n4:upper8bits 
\                        t/f:true if there is i2c-error
: rd_seq
\ Start I2C 
_eestart
\ Write slave address[wr], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)   
tuck _eewrite                 \ ( n2 n1 t/f )
\ Write register
swap_wr                       \ ( n2 t/f )
swap                          \ ( t/f n2 )
\ Start read_process
Sr
\ Write slave address[rd], then receive Acknowledge-bit(ACK:Lo  NACK:Hi)
1 or _eewrite or              \ ( t/f )
\ Read 1byte 
0 _eeread                     \ ( t/f n3 )
\ Read 1byte ,then set sda to Hi(NACK:master->slave)
-1 _eeread                    \ ( t/f n3 n4 )
\ Stop I2C
_eestop
rot                           \ ( n3 n4 t/f )
;


\ Read data[16bits] to each registers[****A/****B]
\ ( n1 n2 -- )  n1:register n2:16bits[hex]
\ register
\ IODIRA       IODIRA(lower8bits)/IODIRB(upper8bits)  1=input 0=output
\ IPOLA        IPOLA(lower8bits)/IPOLB(upper8bits)  1=oppsitelogic state 0=same logic state
\ GPINTENA     GPINTENA(lower8bits)/GPINTENB(upper8bits)  1=enable 0=disable
\ DEFVALA      DEFVALA(lower8bits)/DEFVALB(upper8bits)  
\ INTCONA      INTCONA(lower8bits)/INTCONB(upper8bits)    
\              1=compared for interupt-on-change 0=compared against previous value
\ IOCON        IOCONA(lower8bits)/IOCONB(upper8bits)
\ GPPUA        GPPUA(lower8bits)/GPPUB(upper8bits)  1=Pull-up enabled  0= Pull-up disabled
\ INTFA        INTFA(lower8bits)/INTFB(upper8bits)  1=Pin cause interupt  0= interupt not pending
\              READ-ONLY
\ INTCAPA      INTCAPA(lower8bits)/INTCAPB(upper8bits)  1=logic-hi  0=logic-lo
\ GPIOA        GPIOA(lower8bits)/GPIOB(upper8bits)      1=logic-hi  0=logic-lo
\ OLATA        OLATA(lower8bits)/OLATB(upper8bits)      1=logic-hi  0=logic-lo
\ ( n1 -- )  n1:register
: rd_MCP23017
dup                                \ ( n1 n1 )          
MCP23017                           \ ( n1 n1 slave-address) 
rd_seq
err?                               \ ( n1 n2 n3 )  n1:register n2:lower8bits n3:upper8bits
swap                               \ ( n1 n3 n2 )
rot dup                            \ ( n2 n3 n1 n1)
hex
reg_name dispStr ." :h" swap . cr   \ ( n2 n1 )
1+ reg_name dispStr ." :h" . cr      \ ( )
decimal
;


